/*******************************************************************************
 *	BDspFifo.h	DSP FIFO Buffer Class
 *			T.Barnaby,	BEAM Ltd,	2005-11-18
 *******************************************************************************
 */
#ifndef BDspFifo_H
#define BDspFifo_H	1

#include <BDspMain.h>
#include <BError.h>
#include <BCondInt.h>
#include <BMutex.h>

class BDspFifo;
class BDspFifoReader;

/// This class implements a pointer into the Fifo's circular buffer
class BDspFifoPos {
public:
			BDspFifoPos(BUInt size);
	void		setSize(BUInt size);

	void		set(BUInt pos);				///< Sets the position
	BUInt		pos();					///< The current position
	
	void		increment(BUInt numSamples);		///< Increment the pointer by the given value
	BUInt		difference(const BDspFifoPos& pos);	///< Return the difference between the two pointers

			operator int();
	void		operator+=(BUInt numSamples);
	int		operator==(const BDspFifoPos& pos);
	int		operator!=(const BDspFifoPos& pos);
private:
	BUInt		osize;
	BUInt		opos;
};

/// This class implements a thread safe FIFO buffer
class BDspFifo {
public:
	enum		{ defaultSize = 1024 };

			BDspFifo(BUInt size = defaultSize);
			~BDspFifo();

	BUInt		size();						///< Return the buffers actual size
	void		clear();					///< Clear all of the data in the buffer

			// Data operation functions
	BUInt		writeAvailable();				///< Returns the space available to write
	BError		writeWaitAvailable(BUInt numSamples);		///< Wait for the given number of samples
	
	BError		write(const BDspSample* data, BUInt numSamples);///< Writes the data to the buffer. Blocks until complete

			// Data write functions
	BDspSample*	writeData();					///< Return a pointer to the current start of the buffer
	void		writeDone(BUInt numSamples);			///< Update the write pointer

	BDspSample&	operator[](int pos);				///< Direct access to write samples in buffer
	
	BUInt		readAvailable();				///< Retunr sthe number of items in the Fifo
	BError		readWaitAvailable(BUInt numSamples);		///< Wait for the given number of samples to be available
	void		readerAdd(BDspFifoReader* reader);			///< Adds a BDspFifo Reader
	void		readerRemove(BDspFifoReader* reader);		///< Removes a BDspFifo Reader

	// Functions for internal use
	BError		readDone(BDspFifoReader* reader, BUInt numSamples);		///< Updates read pointer for main fifo
	BDspFifoPos	readPos();
	BDspFifoPos	writePos();
	BDspSample*	data();
	
private:
	BError			mapCircularBuffer(BUInt size);
	void			unmapCircularBuffer();

	BMutex			olock;
	BUInt			ovmSize;
	BUInt			osize;
	BDspSample*		odata;
	BDspFifoPos		owritePos;				///< Current write position
	BCondValue		owriteNumSamples;			///< The number of samples in the FIFO
	BDspFifoPos		oreadPos;				///< Current read position
	BList<BDspFifoReader*>	oreaders;				///< The readers
};

/// This class implements a thread safe FIFO reader
class BDspFifoReader {
public:
			BDspFifoReader(BDspFifo& fifo);
			~BDspFifoReader();

	void		clear();
	
	BUInt		readAvailable();				///< Returns the number of bytes of data available
	BError		readWaitAvailable(BUInt numSamples);		///< Wait for given number of samples
	
	BError		read(BDspSample* data, BUInt numSamples);
	
	BDspSample*	readData();					///< Pointer to raw data
	BError		readDone(BUInt numSamples);			///< Updates read pointer

	BDspSample&	operator[](int pos);

	BDspFifoPos	fifoReadPos();					///< Returns current read position
	void		written(BUInt numSamples);			///< The number of samples written to fifo

private:
	BMutex		olock;
	BDspFifo&	ofifo;
	BDspFifoPos	ofifoReadPos;					///< Current read position in fifo
	BCondValue	oreadNumSamples;
};

/// This class implements a generic DSP processor
class BDspFifoProcess {
public:
				BDspFifoProcess(BDspFifo& input, BDspFifo& output);
	virtual			~BDspFifoProcess();
	virtual BError		process(BUInt numSamples);

protected:
	BDspFifoReader		oinputReader;
	BDspFifo&		ooutput;
};
#endif
