/*******************************************************************************
 *	BdsD.h	Produced by Bidl
 *******************************************************************************
 */
/// @file BdsD.h
/// @date 2023-07-26T16:24:59
/// @brief BOAP data class definitions for: Bds
/// @details The classes in here have been defined by a BOAP *.bidl file and
///   define classes able to be communicated across a BOAP link

#ifndef BDSD_H
#define BDSD_H 1

#include <Boap.h>
#include <BObj.h>
#include <BDate.h>
#include <BTimeStamp.h>
#include <BComplex.h>
#include <BList.h>
#include <BArray.h>

//*****************************************************************************/
//	BDS API
//			T.Barnaby,	BEAM Ltd,	2021-05-19
//*****************************************************************************/
//
namespace Bds {
	/// The System Error number list in addition to standard system error numbers.
	enum Errors 	{ ErrorNoMetaData = 64, ErrorDataQuality = 65, ErrorSlaveMode = 66, ErrorTimeStamp = 67, ErrorValidate = 80, ErrorValidateMissingBlocks = 81, ErrorValidateTimeBackwards = 82, ErrorValidateFilenameTime = 83, ErrorValidateMetaData = 84, ErrorValidateFix = 85, ErrorValidateDuplicate = 86, ErrorValidateReorder = 87, ErrorValidateBdsFudge = 88};

	/// Priority levels
	enum Priority 	{ PriorityLow, PriorityNormal, PriorityHigh};

	/// BdsServer mode
	enum Mode 	{ ModeMaster, ModeSlave};

	/// Flags when opening data files
	enum DataFlags 	{ DataFlagNone = 0x00, DataFlagClipDataToTime = 0x01, DataFlagClipDataToChannels = 0x02, DataFlagMergeSegments = 0x04, DataFlagNoMetadata = 0x08};

	/// The Selection group when making selections
	enum SelectionGroup 	{ SelectionGroupData, SelectionGroupMetaData, SelectionGroupDataWithCount};

	/// The actual format of a data sample
	enum SampleFormat 	{ SampleFormatUnknown, SampleFormatInt16, SampleFormatInt32, SampleFormatFloat32, SampleFormatFloat64, SampleFormatInt24};

	/// A flag defining the data availability state
	enum AvailType 	{ AvailNone, AvailPartial, AvailFull};

	/// Data format abilities bitset
	enum DataFormatSet 	{ DataFormatSetNone = 0x00, DataFormatSetMetadataRead = 0x01, DataFormatSetMetadataWrite = 0x02, DataFormatSetSensordataRead = 0x04, DataFormatSetSensordataWrite = 0x08};

	/// Which Locations to select
	enum LocationSelect 	{ LocationSelectAll, LocationSelectStation, LocationSelectChannel};

	/// @brief This class defines an X,Y location.
	/// @details The class simply stores the x and y point values.
	class Point {
	public:
		Point(BFloat64 x = 0, BFloat64 y = 0);
	public:
		BFloat64	x;
		BFloat64	y;
	};

	/// @brief This class defines a TimePeriod.
	/// @details It has startTime and endTime fields. Note the endTime is just after the actual period.
	class TimePeriod : public BObj {
	public:
		TimePeriod(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BTimeStamp	startTime;	///< The Start time to the nearest us
		BTimeStamp	endTime;	///< The End time to the nearest us
	};

	/// @brief This class defines an integer based range.
	/// @details It is used for limit the number of items returned in selections etc.
	class ListRange : public BObj {
	public:
		ListRange(BUInt32 start = 0, BUInt32 number = 0, BInt32 reverse = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	start;	///< The start position
		BUInt32	number;	///< The number of items
		BInt32	reverse;	///< List from end
	};

	/// @brief This class defines a seismic Network organisation.
	/// @details Typical Seismic Networks are "BN", IDC" etc. The stations list is depreciated.
	class Network : public BObj {
	public:
		Network(BUInt32 id = 0, BString network = BString(), BString description = BString(), BList<BString > stations = BList<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BString	network;	///< The name
		BString	description;	///< The organisations description
		BList<BString >	stations;	///< The list of arrays/stations the Network uses. (depreciated)
	};

	/// @brief This class defines a seismic data Source.
	/// @details A Seismic data source allows different sources of data to be described and allows different Metadata sets
	/// to be used with the different data sources. It might be that there were two different digitisers in use on a Channel
	/// or one data set was received real-time though a particular data processing chain while the other was via CD medium with
	/// a different processing chain.
	class Source : public BObj {
	public:
		Source(BUInt32 id = 0, BString source = BString(), BString sourceMeta = BString(), BString alias = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BString	source;	///< The sensor data's source name
		BString	sourceMeta;	///< The associated metadata's source name
		BString	alias;	///< The short alias for data files
		BString	description;	///< The description
	};

	/// @brief This class defines a Source Priority entry.
	/// @details This allows the default source for data to be selected based on a priority level.
	/// It allows a particular source to be used if no other is available prioritised through
	/// all the different sources available.
	class SourcePriority : public BObj {
	public:
		SourcePriority(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString source = BString(), BUInt32 priority = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	source;	///< The source name
		BUInt32	priority;	///< The priority order, highest first
	};

	/// @brief This class defines a full channel name.
	/// @details A channel's data is fully defined by the Network:Station:Channel:Source.
	/// This class stores all of these component names.
	class ChannelName {
	public:
		ChannelName(BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString());
	public:
		BString	network;	///< The Channels network
		BString	station;	///< The Channels station.
		BString	channel;	///< The Channels name
		BString	source;	///< The Channels source.
	};

	/// @brief This class defines an array's channel.
	/// @details A Station could be an individual station or an array of stations.
	/// If it is an array of stations the Station object contains the list of Stations and Channels that make
	/// up the array.
	class ArrayChannel {
	public:
		ArrayChannel(BString network = BString(), BString station = BString(), BString channel = BString(), BFloat64 arrayOffsetEast = 0, BFloat64 arrayOffsetNorth = 0);
	public:
		BString	network;	///< The Network this station belongs to if for a partricular network
		BString	station;	///< The Stations name.
		BString	channel;	///< The Channels name
		BFloat64	arrayOffsetEast;	///< The Location offset in in an array in an easterly direction in metres.
		BFloat64	arrayOffsetNorth;	///< The Location offset in in an array in a northerly direction in metres.
	};

	/// @brief This class defines a seismic station.
	/// @details A Station could be an individual station or an array of stations.
	/// If it is an array of stations (type is set to "array") them the Station object contains the list of chaannels that make
	/// up the array. The list of channels contain Location offsets to to the individual channels relative to the arry's centre.
	/// This class defines a Station
	class Station {
	public:
		Station(BUInt32 id = 0, BString network = BString(), BString name = BString(), BString alias = BString(), BString type = BString(), BString description = BString(), BList<ArrayChannel > channels = BList<ArrayChannel >());
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BString	network;	///< The Network this station belongs to if for a partricular network
		BString	name;	///< The name
		BString	alias;	///< Alias name to be returned to the user
		BString	type;	///< The Station type. Set to "array" or "station".
		BString	description;	///< Description
		BList<ArrayChannel >	channels;	///< List of channels if an Array.
	};

	/// @brief This class defines the physical location of a Station.
	/// @details This defines the physical location of the station using WGS84 longitude and latitude parameters.
	/// It also defines the stations elevation.
	/// If it is part of an array the arrayOffsetEast and arrayOffsetNorth will be filled in from the Array's ArrayChannel offset information.
	/// So although arrayOffsetEast and arrayOffsetNorth are part of a Location object they are not stored in the database StationLocations table.
	class Location : public BObj {
	public:
		Location(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString datum = BString(), BFloat64 longitude = 0, BFloat64 latitude = 0, BFloat64 elevation = 0, BFloat64 arrayOffsetEast = 0, BFloat64 arrayOffsetNorth = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station this location is for
		BString	channel;	///< The channel this location is for. If blank the location is for the Station itself.
		BString	datum;	///< The locations Datum
		BFloat64	longitude;	///< The longitude in degrees using the WGS84 datum.
		BFloat64	latitude;	///< The Latitude in degrees using the WGS84 datum.
		BFloat64	elevation;	///< The ground level elevation in meters from the WGS84 ellipsoid (Sea level)
		BFloat64	arrayOffsetEast;	///< The Array offset in an array in an easterly direction if array known/given. Note returns Array's ArrayChannel offset information.
		BFloat64	arrayOffsetNorth;	///< The Array offset in an array in a northerly direction if array known/given. Note returns Array's ArrayChannel offset information.
	};

	//
	/// @brief This class defines a Pole/Zero Response
	/// @details It consits of an array of Complex poles and an array of Complex zeros.
	class PoleZero {
	public:
		PoleZero(BArray<BComplex > poles = BArray<BComplex >(), BArray<BComplex > zeros = BArray<BComplex >());
	public:
		BArray<BComplex >	poles;	///< Poles
		BArray<BComplex >	zeros;	///< Zeros
	};

	//
	/// @brief This class defines an entry in an Amplitude/Phase Response table
	class Fap {
	public:
		Fap(BFloat64 frequency = 0, BFloat64 amplitude = 0, BFloat64 phase = 0);
	public:
		BFloat64	frequency;	///< The frequency
		BFloat64	amplitude;	///< The Amplitude
		BFloat64	phase;	///< The Phase
	};

	//
	/// @brief This class defines an entry in a FIR coefficient table.
	class FirEntry {
	public:
		FirEntry(BFloat64 coefficient = 0, BFloat64 error = 0);
	public:
		BFloat64	coefficient;	///< FIR Coefficient Value
		BFloat64	error;	///< FIR Coefficien Error
	};

	//
	/// @brief This class defines an FIR response table.
	/// @details This has an array of the A and B coefficients.
	class Fir {
	public:
		Fir(BArray<FirEntry > b = BArray<FirEntry >(), BArray<FirEntry > a = BArray<FirEntry >());
	public:
		BArray<FirEntry >	b;	///< FIR Numerator array
		BArray<FirEntry >	a;	///< FIR Denominator array
	};

	//
	/// @brief This class defines an entry in a Polynomial coefficient table.
	class PolynomialEntry {
	public:
		PolynomialEntry(BFloat64 coefficient = 0, BFloat64 plusError = 0, BFloat64 minusError = 0, BString measurementMethod = BString());
	public:
		BFloat64	coefficient;	///< The coefficient
		BFloat64	plusError;	///< Plus uncertainty or error in measured value
		BFloat64	minusError;	///< Minus uncertainty or error in measured value
		BString	measurementMethod;	///< The measurement method
	};

	//
	/// @brief This class defines an Polynomial response table.
	/// @details This has an array of the polynomial coefficients.
	class Polynomial {
	public:
		Polynomial(BString transferType = BString(), BString approximationType = BString(), BString validFrequencyUnits = BString(), BFloat64 frequencyLowerBound = 0, BFloat64 frequencyUpperBound = 0, BFloat64 approximationLowerBound = 0, BFloat64 approximationUpperBound = 0, BFloat64 maximumError = 0, BArray<PolynomialEntry > coefficients = BArray<PolynomialEntry >());
	public:
		BString	transferType;	///< Transfer Type
		BString	approximationType;	///< Polynomial Approximation Type
		BString	validFrequencyUnits;	///< Valid Frequency Units
		BFloat64	frequencyLowerBound;	///< Lower Valid Frequency Bound
		BFloat64	frequencyUpperBound;	///< Upper Valid Frequency Bound
		BFloat64	approximationLowerBound;	///< Lower Bound of Approximation
		BFloat64	approximationUpperBound;	///< Upper Bound of Approximation
		BFloat64	maximumError;	///< Maximum Absolute Error
		BArray<PolynomialEntry >	coefficients;	///< The Coeficients
	};

	//
	/// @brief This class defines a seismic Response characteristic.
	/// @details For each seismic channel there is a frequency response characteristic.
	/// There can be multiple stages in a channels frequency response, this response data describes one
	/// of those stages frequencies responses. The stage parameter defines which stage it is for (1, 2, 3, ...)
	/// Stage 0 is reserved to store an overall channel response for special puposes.
	/// A response can be in the form of an array of poles and zeros, a FAP array, a set of FIR coefficients or a Polynomial.
	/// This object contains Metadata for other responce characteristics as defined in various seismic Response formats.
	/// In general this extra Metadata is simply passed through the BDS and is used when exporting Metadat in different formats.
	/// Note that the actual response data is stored in a database Blob and the poleZeros, faps, fir or polynomial fields are filled in based upon the Response type.
	class Response {
	public:
		Response(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString(), BUInt32 stage = 0, BString name = BString(), BString type = BString(), PoleZero poleZeros = PoleZero(), BArray<Fap > faps = BArray<Fap >(), Fir fir = Fir(), Polynomial polynomial = Polynomial(), BFloat64 gain = 0, BFloat64 gainFrequency = 0, BString stageType = BString(), BFloat64 decimation = 0, BFloat64 decimationOffset = 0, BFloat64 decimationDelay = 0, BFloat64 decimationCorr = 0, BString symmetry = BString(), BString description = BString(), BInt32 measured = 0, BFloat64 sampleRate = 0, BString inputUnits = BString(), BString inputUnitsDesc = BString(), BString outputUnits = BString(), BString outputUnitsDesc = BString());
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The start time when ret response is valid from
		BTimeStamp	endTime;	///< The end time the response is valid to
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station
		BString	channel;	///< The channel
		BString	source;	///< The source
		BUInt32	stage;	///< The stage (0, 1, 2, 3, ...). 0 is a special overall response sometimes used.
		BString	name;	///< The response name. (Anti-Aliasing filter, Digitiser, post filter etc)
		BString	type;	///< The type of response (PoleZero, FIR, FAP, Polynomial or blank if no frequency response)
		PoleZero	poleZeros;	///< The PoleZeros
		BArray<Fap >	faps;	///< The FAP Frequency/Amplitude/Phase table
		Fir	fir;	///< The FIR filters coefficients
		Polynomial	polynomial;	///< The polynomial response
		BFloat64	gain;	///< Overall gain at gainFrequency.
		BFloat64	gainFrequency;	///< Frequency that gain is valid for.
		BString	stageType;	///< The stage type: A - Analog (rad/sec), B - Analog (Hz), C - Composite, D - Digital
		BFloat64	decimation;	///< Decimation factor performed post filter
		BFloat64	decimationOffset;	///< Decimation sample offset
		BFloat64	decimationDelay;	///< Decimation delay in seconds
		BFloat64	decimationCorr;	///< Decimation correction performed
		BString	symmetry;	///< Symmetry for FIR coefficients (A = asymmetric, B = symmetric[odd], C = symmetric[even])
		BString	description;	///< Misc description
		BInt32	measured;	///< Boolean if response was a measured response
		BFloat64	sampleRate;	///< The stage's sample rate
		BString	inputUnits;	///< The units of the data as input from the perspective of data acquisition. After correcting data for this response, these would be the resulting units.
		BString	inputUnitsDesc;	///< The input units description
		BString	outputUnits;	///< The units of the data as output from the perspective of data acquisition. These would be the units of the data prior to correcting for this response.
		BString	outputUnitsDesc;	///< The output units description
	};

	/// @brief This class defines a calibration setting.
	/// @details Each channel has the core information of a samplingFrequency and a calibrationFactor (scaling factor) associated with it at a particular calibrationFrequency.
	/// The calibrationUnits defeines the units of the senor data after all of the Response frequency response's have been applied. Responses could add integration stages that
	/// will change the original senor dataors units to the given calibrationUnits.
	/// The raw* parameters, if given, are the raw sensor data's parameters before the Responses are applied. This is useful for simple raw data graph ploting etc.
	/// There may be additional calibration information such as the depth of the sensor and its positional angles.
	class Calibration : public BObj {
	public:
		Calibration(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString(), BString name = BString(), BFloat64 samplingFrequency = 0, BFloat64 calibrationFrequency = 0, BFloat64 calibrationFactor = 0, BString calibrationUnits = BString(), BString calibrationUnitsDesc = BString(), BFloat64 rawCalibrationFrequency = 0, BFloat64 rawCalibrationFactor = 0, BString rawCalibrationUnits = BString(), BFloat64 depth = 0, BFloat64 waterLevel = 0, BFloat64 horizontalAngle = 0, BFloat64 verticalAngle = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network/Organisation Name
		BString	station;	///< The station
		BString	channel;	///< The channel
		BString	source;	///< The source
		BString	name;	///< The Calibrations name, "Main", "Measured".
		BFloat64	samplingFrequency;	///< The sample rate used in Hz
		BFloat64	calibrationFrequency;	///< The post response compensation frequency that the CalibrationFactor value is valid for in Hz
		BFloat64	calibrationFactor;	///< The post response compensation scaling value to apply to the data to normalise data to the units. This is a measured value at the calibration frequency.
		BString	calibrationUnits;	///< The post response compensation measurment SI units such as "m".
		BString	calibrationUnitsDesc;	///< A description of the calibrationUnits
		BFloat64	rawCalibrationFrequency;	///< The raw data frequency that the CalibrationFactor value is valid for in Hz
		BFloat64	rawCalibrationFactor;	///< The raw data scaling value to apply to the data to normalise data to the units. This is a measured value at the calibration frequency.
		BString	rawCalibrationUnits;	///< The raw data measurment SI units such as "m".
		BFloat64	depth;	///< The depth of the sensor below ground level in meters
		BFloat64	waterLevel;	///< Elevation of the water surface in meters for underwater sites, where 0 is sea level.
		BFloat64	horizontalAngle;	///< The Sensors channel placement horizontal angle in degrees clockwise from north.
		BFloat64	verticalAngle;	///< The Sensors channel placement vertical angle in degrees degrees with zero = vertically up.
	};

	/// @brief This class defines a seismic Digitiser
	/// @details This just stores information on the seismic instrument's digitiser. Its contents is generally for information only.
	class Digitiser : public BObj {
	public:
		Digitiser(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString name = BString(), BString type = BString(), BString serialNumber = BString(), BUInt32 numberChannels = 0, BFloat64 baseSamplingFrequency = 0, BFloat64 initialSamplingFrequency = 0, BFloat64 gain = 0, BInt32 shared = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	name;	///< The Digitisers name
		BString	type;	///< The Digitisers type
		BString	serialNumber;	///< The digitisers's serial number.
		BUInt32	numberChannels;	///< The number of supported channels
		BFloat64	baseSamplingFrequency;	///< The base sampling frequency
		BFloat64	initialSamplingFrequency;	///< The initial pre-decimation sampling frequency
		BFloat64	gain;	///< The overall gain of the digitiser at the manufacturers calibration frequency. (For information only)
		BInt32	shared;	///< This digitiser is shared
	};

	/// @brief This class defines a seismic Sensor
	/// @details This just stores information on the seismic instrument's senosr. Its contents is generally for information only.
	class Sensor : public BObj {
	public:
		Sensor(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString name = BString(), BString type = BString(), BString serialNumber = BString(), BUInt32 numberChannels = 0, BString gainUnits = BString(), BFloat64 gain = 0, BUInt32 oldId = 0, BInt32 shared = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The ID
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	name;	///< The Sensors name
		BString	type;	///< The type of sensor. (Seismometer, Hydrophone etc)
		BString	serialNumber;	///< The sensor's serial number. Only used when there is a unique physical sensor.
		BUInt32	numberChannels;	///< The number of supported channels
		BString	gainUnits;	///< The gain units
		BFloat64	gain;	///< The overall gain of the sensor at the manufacturers calibration frequency. (For information only)
		BUInt32	oldId;	///< The Id from the old Autodrm database
		BInt32	shared;	///< This sensor is shared
	};

	/// @brief This class defines a Channel's instrument.
	/// @details This ChannelInstrument links a seismic data channel with a paricular sensor and digitiser.
	/// Note that it is possible to share sensors and digitisers between channels if wanted for generic sensor/digitiser definitions.
	/// However if particular serial numbers are needed the sensor/digitiser needs to be unique.
	class ChannelInstrument : public BObj {
	public:
		ChannelInstrument(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BUInt32 channelId = 0, BString source = BString(), BUInt32 digitiserId = 0, BUInt32 sensorId = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BUInt32	channelId;	///< The channels Id
		BString	source;	///< The source
		BUInt32	digitiserId;	///< The Digitiser in use
		BUInt32	sensorId;	///< The sensor in use
	};

	/// @brief This class defines a seismic data Channel.
	/// @details This class defines a seismic data channel with network:station:channel definitions.
	/// The class also splits the channel's name field into channelType and channelAux (channel name is <channelType>_<channelAux> to
	/// allow easy database searches etc. Note that the channelType and channelAux fields are calculated from the channel field and should not be changed.
	/// As well as seismic data, a Channel can contain other data types.
	class Channel : public BObj {
	public:
		Channel(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString channelType = BString(), BString channelAux = BString(), BString dataType = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network Name
		BString	station;	///< The Stations name.
		BString	channel;	///< The channels name (often as <channelType>_<channelAux>)
		BString	channelType;	///< The channels type (component of station field)
		BString	channelAux;	///< The channels auxilary identifier (component of station field)
		BString	dataType;	///< The Type of data (seismic, seismicUnknown, data, log, unknown, empty)
		BString	description;	///< The channels description
	};

	/// @brief This class defines the set of Metadata or Siesmic sensor data to be selected when getSelectionInfo() is use.
	/// @details This provides information on everything selected by a Selection object from the BDS Metadata or Sensor data sets.
	class SelectionInfo {
	public:
		SelectionInfo(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BList<BString > networks = BList<BString >(), BList<BString > arrays = BList<BString >(), BList<BString > stations = BList<BString >(), BList<BString > arraysAndStations = BList<BString >(), BList<BString > channels = BList<BString >(), BList<BString > sources = BList<BString >(), BUInt32 numDataChannels = 0);
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BList<BString >	networks;	///< The list of Network Names
		BList<BString >	arrays;	///< The list of Array names
		BList<BString >	stations;	///< The list of Station names
		BList<BString >	arraysAndStations;	///< The list of Array and Station names
		BList<BString >	channels;	///< The list of Channels
		BList<BString >	sources;	///< The list of Data Sources
		BUInt32	numDataChannels;	///< The number of sets of data in the system matching the criteria
	};

	/// @brief This class defines an idividual channel for selection.
	/// @details It contains the network:station:channel:source names.
	class SelectionChannel {
	public:
		SelectionChannel(BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString());
	public:
		BString	network;
		BString	station;
		BString	channel;
		BString	source;
	};

	/// @brief This class defines a generic Metadata or Sensor data selection.
	/// @details This defines a set of selection criteria when selecting items from the BDS Metadata or Sensor data sets.
	/// The fields, when set, limit the items returned as defined by the setting fields provided. In effect these have an "AND" type of function
	/// with the parameters provided. Most of the string parameters can be provided as regular expressions.
	/// The core section fields are the startTime, endTime and channels fields.
	/// As well as thses there are some specific parameters like: channelId, digitiserId, sensorId etc which are used with specific
	/// data selection functions.
	class Selection {
	public:
		Selection(BUInt32 id = 0, ListRange range = ListRange(), BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BList<SelectionChannel > channels = BList<SelectionChannel >(), BUInt32 channelId = 0, BUInt32 digitiserId = 0, BUInt32 sensorId = 0, BUInt32 sensorOldId = 0, BInt32 completeSegments = 0, BString calibrationName = BString(), BString array = BString(), BUInt32 eventId = 0, BString name = BString(), LocationSelect locationSelect = LocationSelect(), BString dataTypes = BString(), BString excludeChannels = BString());
	public:
		BUInt32	id;	///< The ID of the record to return
		ListRange	range;	///< The range of data to return
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BList<SelectionChannel >	channels;	///< The data channels to select
		BUInt32	channelId;	///< The Channel id
		BUInt32	digitiserId;	///< The Digitiser id
		BUInt32	sensorId;	///< The Sensor id
		BUInt32	sensorOldId;	///< The Sensor old id
		BInt32	completeSegments;	///< Do not clip the segment times to match the required time period
		BString	calibrationName;	///< Calibration name to use
		BString	array;	///< Channels are based on the given array (for array offsets)
		BUInt32	eventId;	///< A particular event's ID
		BString	name;	///< Match the name, title or some other string in the objects to select
		LocationSelect	locationSelect;	///< Which locations to select, those for stations, channels or all of them when used in a locationGetList() call.
		BString	dataTypes;	///< Place holder as yet for: A comma separated list of Channel dataTypes to select when returning Channels. If set to null all Channel dataTypes are returned
		BString	excludeChannels;	///< Comma separated wildcard list of channels to be excluded
	};

	/// @brief This class provides full Metadata information on a channel
	/// @details This returns the Metadata available for a channel over a particular time period.
	/// There are likely to be multiple ChannelInfo objects over larger time periods, one for each change in Metadata.
	/// The ChannelInfos object contains an array of these ChannelInfo objects.
	class ChannelInfo {
	public:
		ChannelInfo(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), Station station = Station(), Location stationLocation = Location(), Channel channel = Channel(), Location channelLocation = Location(), BString source = BString(), Digitiser digitiser = Digitiser(), Sensor sensor = Sensor(), Calibration calibration = Calibration(), BList<Response > responses = BList<Response >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		Station	station;	///< The Station info
		Location	stationLocation;	///< The Station location
		Channel	channel;	///< The Channel data
		Location	channelLocation;	///< The Channel location
		BString	source;	///< The data source
		Digitiser	digitiser;	///< The Digitiser in use
		Sensor	sensor;	///< The Sensor in use
		Calibration	calibration;	///< The Calibration info
		BList<Response >	responses;	///< The list of frequency responses
	};

	/// @brief This class provides Metadata information on a set of channels
	/// @details This provides all of the Metadata information for a set of channels over a given time period.
	/// For each channel there can be one or more ChannelInfo objects each defining a set of consistant Metadata
	/// over a particular time period. The BdsServer will create a separate ChannelInfo object on each change in Metadata
	/// when the user asks for a set of metadata over some time period.
	class ChannelInfos {
	public:
		ChannelInfos(Station array = Station(), BArray<BArray<ChannelInfo > > channels = BArray<BArray<ChannelInfo > >());
	public:
		Station	array;	///< The array if all channels are from an array
		BArray<BArray<ChannelInfo > >	channels;	///< The per channel per time segment Metadata. For each channel there can be one or more ChannelInfo objects. A new ChannelInfo object is introduced whenever an item of Metadata changes.
	};

	/// @brief This class defines information on a Sensor data file
	/// @details The raw sensor data for a seismic channel is stored in files in the BDS system.
	/// This class defines the database entry that describes this file, its storage location and status.
	/// A single file can store one or more channels of seismic data in different formats although noramlly the BDS data file format is used.
	class DataFileInfo : public BObj {
	public:
		DataFileInfo(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString location = BString(), BString format = BString(), BString url = BString(), BString stream = BString(), BString comment = BString(), BUInt32 importUserId = 0, BTimeStamp importTime = BTimeStamp(), BString state = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	location;	///< The storage location
		BString	format;	///< The data format
		BString	url;	///< The URL for file access
		BString	stream;	///< The real-time data stream
		BString	comment;	///< A comment on the file
		BUInt32	importUserId;	///< The user ID of the importing user
		BTimeStamp	importTime;	///< The Time the data was imported
		BString	state;	///< Status info on the import (importing, realtime, failed, ok etc)
	};

	/// @brief This class defines information on a single channel's set of data stored in a file.
	/// @details This provides information on actual Sensor data for a channel that is stored in the BDS system.
	/// The data will be stored in a particular file perhaps with other data channels (DataFileInfo).
	/// When known, information on the channels numBlocks, numSamples and sampleRate will be provided. Generally this information
	/// will only be known if a data file has been imported rather than a live real-time data stream.
	/// Generally the Sensor data file itself should be interrogated to find the definitive information.
	/// The info field provides extra details on the data contents which might come from one of the specific data import formats.
	class DataChannel : public BObj {
	public:
		DataChannel(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString(), BUInt32 numBlocks = 0, BUInt64 numSamples = 0, BFloat64 sampleRate = 0, BUInt32 sampleFormat = 0, BUInt32 dataFileId = 0, BUInt32 dataFileChannel = 0, BString importFormat = BString(), BString importFilename = BString(), BTimeStamp importStartTime = BTimeStamp(), BDict<BString > info = BDict<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BUInt32	numBlocks;	///< The total number of blocks per channel if known, 0 otherwise
		BUInt64	numSamples;	///< The total number of samples per channel if known, 0 otherwise
		BFloat64	sampleRate;	///< The data's sample rate
		BUInt32	sampleFormat;	///< The data sample format
		BUInt32	dataFileId;	///< The Data File Id. This links to the particular DataFileInfo where the data is stored.
		BUInt32	dataFileChannel;	///< The Data File Channel number. The channel number within the data file. (1, 2, 3 ...)
		BString	importFormat;	///< The original data format
		BString	importFilename;	///< The original data file name
		BTimeStamp	importStartTime;	///< The original import files start time
		BDict<BString >	info;	///< Extra info on the channel
	};

	/// @brief This class defines information on a set of data.
	/// @details This describes a set of Sensor data. It is used to returning basic Sensor data information when performing a selection of data
	/// or detailed information when enquiring information from an actual Sensor data file.
	/// All of the detailed information comes from the Sensor data files themselves. This includes the info, infoExtra and warnings information.
	/// The channels array contains an array of data segments per channel split at time discontinuities. For basic information this could be a single segment over
	/// a time period. However, when equiring detailed information from a file it will contain an entry per contiguous data segemnt
	/// in the file.
	class DataInfo {
	public:
		DataInfo(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString array = BString(), BString description = BString(), BUInt32 synchronous = 0, BArray<BArray<DataChannel > > channels = BArray<BArray<DataChannel > >(), BDict<BString > info = BDict<BString >(), BDict<BString > infoExtra = BDict<BString >(), BList<BString > warnings = BList<BString >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	array;	///< The Seismic Array that all of the channels are from, if just one
		BString	description;	///< The Comment
		BUInt32	synchronous;	///< The channels are synchronously sampled
		BArray<BArray<DataChannel > >	channels;	///< The Data channels. Each channel can have multiple segments of data
		BDict<BString >	info;	///< Info on the set of channels
		BDict<BString >	infoExtra;	///< Extra Info on the set of channels. Used for extended error/logging information
		BList<BString >	warnings;	///< Warnings on the data set
	};

	/// @brief This class provides availability information on a particular period of data.
	class DataAvail {
	public:
		DataAvail(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), AvailType availType = AvailType());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		AvailType	availType;	///< The availability type. Can be: AvailNone, AvailPartial or AvailFull.
	};

	/// @brief This class defines avaiilablity information on a set of data.
	/// @details Availability information is provided for a particular channel network:station:channel:source.
	/// For a particular time period this will be broken down into time period segments.
	/// The time granularity of the segments is dependent of the type of data availability search and options in the requests.
	/// Normally the granularity is that defined by the DataChannel information in the database. The actual seismic data files
	/// may not contains some data over the given periods due to missing blocks etc.
	/// An in-depth data availability search could provide more fuller data availability information but with much more data processing,
	/// this has yet to be implemented.
	class DataAvailChan {
	public:
		DataAvailChan(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString(), BArray<DataAvail > segments = BArray<DataAvail >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BArray<DataAvail >	segments;	///< Data time segment availability info
	};

	/// @brief This defines a handle to a sensor data stream/file when opened for read or write.
	class DataHandle {
	public:
		DataHandle(BUInt32 handle = 0, BUInt32 dataFileId = 0);
	public:
		BUInt32	handle;	///< Opaque file handle
		BUInt32	dataFileId;	///< The data file ID if opened for write
	};

	/// @brief This class provides the actual Sensor data values contained within a single data block.
	/// @details All Sensor data is segmented into blocks that have a start and end timestamp and perhaps other metadata.
	/// The DataBlock contains an array of actual data samples in a 64bit floating pointer format for one or more channels.
	/// If the samples are for multiple channels it is assumed that these are synchronously sampled and are normally from
	/// a data file that is in the sample multiplexed format (ie. For each point in time there is a set of samples one for each cahnnel).
	/// The startTime comes from the original blocks start time information.
	/// The endTime may come from the original blocks end time information, if available in the original data format that the data was
	/// imported from or is generated from the startTime and the calculated sample rate of the data.
	/// For some data types where the sampling rate is a bit varible, the endTime fields may be lined up with the next blocks startTime
	/// field to ensure contiguous data segments when exporting data.
	/// The info field contains extra, free string format, metadata on the block if available.
	/// This could be quality information from the TapeDigitiser system for example.
	class DataBlock {
	public:
		DataBlock(BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BUInt32 channelNumber = 0, BUInt32 segmentNumber = 0, BArray<BArray<BFloat64 > > channelData = BArray<BArray<BFloat64 > >(), BDict<BString > info = BDict<BString >());
	public:
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BUInt32	channelNumber;	///< The first channel number. (1, 2, 3 ...)
		BUInt32	segmentNumber;	///< The segment number. (1, 2, 3, ...)
		BArray<BArray<BFloat64 > >	channelData;	///< The raw channel data in a 2 dimensional array, ordered as per channel information in dataInfo
		BDict<BString >	info;	///< Extra information on data or ASCII data
	};

	/// @brief This class provides the actual seismic data values contained within a single data block along with the network:station:channel:source information.
	class DataBlockChannel {
	public:
		DataBlockChannel(BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString());
	public:
		BString	network;	///< The Network Name or names
		BString	station;	///< The Station name or Stations
		BString	channel;	///< The Channels name or Channels
		BString	source;	///< The Data Source or Sources
	};

	/// @brief This holds information on a user
	/// @details All information on a BDS user is stored along with the security groups they belong to.
	class User : public BObj {
	public:
		User(BUInt32 id = 0, BString user = BString(), BString password = BString(), BString name = BString(), BString email = BString(), BString telephone = BString(), BString address = BString(), BInt32 enabled = 0, BList<BString > groups = BList<BString >());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique user ID
		BString	user;	///< The User ID
		BString	password;	///< The User's password
		BString	name;	///< The User's full name
		BString	email;	///< The User's email Address
		BString	telephone;	///< The User's telephone number
		BString	address;	///< The User's postal address
		BInt32	enabled;	///< Whether the users account is enabled
		BList<BString >	groups;	///< The security groups the user belongs to
	};

	/// @brief This holds information on a User security group.
	/// @details The BDS has the concept of a security group that users can belong to.
	/// This class defines that security group as stored in the database.
	class Group : public BObj {
	public:
		Group(BUInt32 id = 0, BString group = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BString	group;	///< The Group name
		BString	description;	///< The Groups description
	};

	/// @brief This holds information on data access groups.
	/// @details A particular network:station may contain sensitive data. This database linked object
	/// links a period of data from a particular network:station to a security group.
	class AccessGroup : public BObj {
	public:
		AccessGroup(BUInt32 id = 0, BString group = BString(), BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BString	group;	///< The Group name
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time
		BString	network;	///< The Network Name
		BString	station;	///< The Station name
	};

	/// @brief This holds information on a Medatdata or Sensor data database change
	/// @details Whenever a change is made to the BDS Metadata or Sensor data a Change object is added to the BDS Changes database.
	/// This describes which database table and object that was added or modified.
	/// Change's are normally grouped together by a ChangeGroup.
	class Change : public BObj {
	public:
		Change(BUInt32 id = 0, BUInt32 changeGroupId = 0, BTimeStamp time = BTimeStamp(), BString type = BString(), BString table = BString(), BUInt32 rowId = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BUInt32	changeGroupId;	///< The Change group ID
		BTimeStamp	time;	///< The Time the change was made
		BString	type;	///< The change type
		BString	table;	///< The database table affected
		BUInt32	rowId;	///< The database row affected
	};

	/// @brief This holds information on a set of Changes
	/// @details A set of changes to the BDS database are grouped into a ChangeGroup.
	/// This could be a set of changes whilst a user is logged in or by a program making
	/// a set of changes.
	class ChangeGroup : public BObj {
	public:
		ChangeGroup(BUInt32 id = 0, BTimeStamp time = BTimeStamp(), BString type = BString(), BString user = BString(), BString title = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	time;	///< The Time the change was made
		BString	type;	///< The type of change
		BString	user;	///< The user who made the change
		BString	title;	///< The Changes title
		BString	description;	///< The Description of the change
	};

	/// @brief This holds information on a Note for general information
	/// @details Normally a Note can be added for a particular set of data over a particular time period.
	/// These notes are sometimes added automaically during a data import process or manually by a user.
	/// A User can then ask for any notes for a particular set of data.
	class Note : public BObj {
	public:
		Note(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString source = BString(), BString type = BString(), BString user = BString(), BTimeStamp timeAdded = BTimeStamp(), BInt32 errorNumber = 0, BString title = BString(), BString description = BString(), BString docFormat = BString(), BString docUrl = BString(), BUInt32 dataFileId = 0, BString importFilename = BString(), BUInt32 eventId = 0);
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	startTime;	///< The Start Time note is for
		BTimeStamp	endTime;	///< The End Time note is for
		BString	network;	///< The Network Name
		BString	station;	///< The Station/Array name
		BString	channel;	///< The Channels name
		BString	source;	///< The Data Source
		BString	type;	///< The Type (note, warning, error ...)
		BString	user;	///< The user
		BTimeStamp	timeAdded;	///< The Time Entered
		BInt32	errorNumber;	///< Error number if error
		BString	title;	///< The title
		BString	description;	///< The Description
		BString	docFormat;	///< Document format if any
		BString	docUrl;	///< Document Url if any
		BUInt32	dataFileId;	///< The data file id associated with this note
		BString	importFilename;	///< The import filename
		BUInt32	eventId;	///< The event ID associated with this note
	};

	/// @brief This holds information on a Log entry.
	/// @details Log entries are added automatically and manually to the system.
	/// A system administartor can view these logs.
	class Log : public BObj {
	public:
		Log(BUInt32 id = 0, BTimeStamp time = BTimeStamp(), BString type = BString(), BUInt32 priority = 0, BString subSystem = BString(), BString title = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< The unique id
		BTimeStamp	time;	///< The Time
		BString	type;	///< The Type
		BUInt32	priority;	///< The priority 0 to 5
		BString	subSystem;	///< The SubSystem
		BString	title;	///< The Changes title
		BString	description;	///< The Description of the change
	};

	/// @brief This defines the selection cirteria when requesting a set of log entries.
	class LogSelect {
	public:
		LogSelect(BTimeStamp startTime = BTimeStamp(), BString type = BString(), BUInt32 priority = 0, BString subSystem = BString());
	public:
		BTimeStamp	startTime;	///< The start time
		BString	type;	///< The Type
		BUInt32	priority;	///< The priority 0 to 5. Will return log entries at or above the priority given
		BString	subSystem;	///< The SubSystem
	};

	/// @brief This defines the set of clean options used in the clean() function.
	class CleanOptions {
	public:
		CleanOptions(BInt32 logs = 0, BInt32 changes = 0, BInt32 deletedFiles = 0);
	public:
		BInt32	logs;	///< Clean the Logs
		BInt32	changes;	///< Clean the changes
		BInt32	deletedFiles;	///< Clean deleted data files
	};

	/// @brief This holds information on a Sensor data format.
	/// @details It is used by the BDS data convertors to define which data formats they support.
	class DataFormat {
	public:
		DataFormat(BList<BString > names = BList<BString >(), BInt32 dataRead = 0, BInt32 dataWrite = 0, BInt32 metadataRead = 0, BInt32 metadataWrite = 0, BString extension = BString(), BString description = BString());
	public:
		BList<BString >	names;	///< The format names
		BInt32	dataRead;	///< Ability to read Sensor data
		BInt32	dataWrite;	///< Ability to write Sensor data
		BInt32	metadataRead;	///< MetaData read supported
		BInt32	metadataWrite;	///< MetaData write supported
		BString	extension;	///< Default filename extension
		BString	description;	///< The description
	};

	/// @brief A list of the available Sensor data formats
	typedef BList<DataFormat > DataFormats;
	
	/// @brief A Special channel identifier
	/// @details This specifies a particular Channel as a special one. The Channels Network,Station and Channel are regular expresions to define one or more channels.
	/// If selected the dataType field is effectively applied to this channel. The dataType "ignore" allows Cahnnels to be ignored on export by default.
	class SpecialChannel : public BObj {
	public:
		SpecialChannel(BUInt32 id = 0, BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BString network = BString(), BString station = BString(), BString channel = BString(), BString dataType = BString(), BString description = BString());
		BString getType();
		BError setMembers(BDictString& members);
		BError setMember(BString name, BString value);
		BError getMembers(BDictString& members);
		BError getMember(BString name, BString& value);
	public:
		BUInt32	id;	///< Unique ID when stored in a database or for other uses.
		BTimeStamp	startTime;	///< The Start Time
		BTimeStamp	endTime;	///< The End Time the channel was available
		BString	network;	///< The Network Name, wildcards allowed
		BString	station;	///< The Stations name, wildcards allowed.
		BString	channel;	///< The channels name, wildcards allowed (often as <channelType>_<channelAux>)
		BString	dataType;	///< The Type of channel (ignore)
		BString	description;	///< The channels description
	};

	/// @brief This class defines a seismic event
	/// @details Each event defines Metadata for a seismic event. This includes a list of all of the data channels relavent to the event.
	/// Given this information the BDS is able, given an Event title, to export all of the Metadata and Sensor data relevent to this event.
	class Event {
	public:
		Event(BUInt32 id = 0, BUInt32 userId = 0, BString type = BString(), BString title = BString(), BString network = BString(), BString source = BString(), BTimeStamp startTime = BTimeStamp(), BTimeStamp endTime = BTimeStamp(), BTimeStamp eventTime = BTimeStamp(), BFloat64 longitude = 0, BFloat64 latitude = 0, BFloat64 elevation = 0, BFloat64 waterDepth = 0, BFloat64 magnitude = 0, BString magnitudeUnits = BString(), BString description = BString(), BString notes = BString(), BDict<BString > extra = BDict<BString >(), BList<SelectionChannel > dataChannels = BList<SelectionChannel >());
	public:
		BUInt32	id;	///< Unique id defining this event within the BDS system
		BUInt32	userId;	///< User ID for initial creator of this event. This allows them to edit these events.
		BString	type;	///< The event type (a hierarchy of types)
		BString	title;	///< Some text describing the event
		BString	network;	///< Unique network for this event if a project.
		BString	source;	///< Unique source for this event if a project.
		BTimeStamp	startTime;	///< The startTime of the event to the nearest microsecond. For data access.
		BTimeStamp	endTime;	///< The endTime of the event to the nearest microsecond. For data access
		BTimeStamp	eventTime;	///< The actual time of the event to the nearest microsecond
		BFloat64	longitude;	///< The longitude in degrees using the WGS84 datum.
		BFloat64	latitude;	///< The Latitude in degrees using the WGS84 datum.
		BFloat64	elevation;	///< The ground level elevation in meters from the WGS84 ellipsoid (Sea level)
		BFloat64	waterDepth;	///< Water depth of the event if in water
		BFloat64	magnitude;	///< Magnitude of the event
		BString	magnitudeUnits;	///< Magnitude Units
		BString	description;	///< General description of the Event/Project
		BString	notes;	///< General notes on the Event/project
		BDict<BString >	extra;	///< An array of name value pairs for extra metadata specific to particular events.
		BList<SelectionChannel >	dataChannels;	///< List of BDS Channels of associated sensor data files if any.
	};

	/// @class DataAccess
	/// @brief This is the Data Access API interface to the BDS system.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the basic and restricted user orientated read only, data access API.
	/// @class DataAddAccess
	/// @brief This is the DataAdd Access API interface.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the basic and restricted user orientated read only, data access API along with the ability
	/// to import data. It will normally be used by data import client programs.
	/// @class AdminAccess
	/// @brief This is the AdminAccess Access API interface.
	/// @details This object provides the set of API functions that make RPC network calls to a BdsServer.
	/// It provides the full unresiticed data access API allowring all data read and write operations to be performed.
}

#endif
