/*******************************************************************************
 *	BdsDataFileSeed.h	SEED Data File Access
 *	T.Barnaby,	BEAM Ltd,	2009-12-18
 *******************************************************************************
 */
#ifndef BdsDataFileSeed_H
#define	BdsDataFileSeed_H

#include <BdsDataFile.h>
#include <BdsSeedTypes.h>
#include <BMutex.h>

typedef struct MSRecord_s MSRecord; // Forward-declaration for libmseed type.

namespace Bds {

/// Data file convertor for SEED file formats
class DataFileSeed : public DataFile{
public:
				DataFileSeed();
				~DataFileSeed();
	
	BError			close();

	DataOrder		getDataOrder();
	int			getFeatures();						///< Get bitmask of supported features
	BString			getFixesInfo();						///< Get readable list of fixes that can be applied to faulty data files
	BError			setFormat(BString format);

				// Read routines
	BError			getInfo(DataInfo& dataInfo, DataFileOptions options, BList<DataError>& errors);
	BError			readData(BUInt32 channel, BUInt segment, BUInt32 blockNumber, DataBlock& data);
	BError			getMetaData(ChannelInfos& channelInfos, BUInt32 options, BList<DataError>& errors);				///< Return all known MetaData in the file

				// Write routines
	BError			setInfo(const DataInfo& dataInfo, const ChannelInfos& channelInfos, WriteOptionsList options = WriteOptionSensorData);
	BError			start(BUInt channel, BUInt segment);
	BError			writeData(const DataBlock& data);
	BError			end();	

	void			msrFileWrite(void* data, int len);

	static DataFormats	getFormats();
	static BError		omsrErr;			///< MSR processing error
	static int		onoLock;			///< Disable libmseed locking

private:
	class NameDesc {
	public:
				NameDesc(BString name = "", BString desc = "") : name(name), desc(desc){}
		BString		name;
		BString		desc;
	};
	
	BError			processControl();
	BError			processBlockette();
	BError			processData(DataBlock& data, int addInfo);
	BError			setBlockSize(BUInt blockSize);
	BError			readBlockData(char* buf, BUInt numBytes);
	BError			readBlock(BUInt64 blockPosition, char& type, char& cont, BUInt& seq);
	BError			writeVolumeHeader(int rl);
	BError			writeTimeSpans();
	BError			writeBlockData(char type, const char* buf, BUInt numBytes);
	BError			writeFlush();
	int			getChannel(BString name);
	void			sortChannels();			///< Sort channels into station order
	DataError		getBlockReorderInfo();
	
	BUInt			addCode(BString str);
	BUInt			addUnitCode(BString name, BString desc);
	Response*		getResponse(BUInt stage);

	DataFileOptions		oreadOptions;
	DataInfo		odataInfo;
	BString			onetwork;
	ChannelInfos		ochannelInfos;
	BArray<ChannelInfo>	ochannelInfoSegments;
	ChannelInfo		ochannelInfo;
	double			osampleRate;
	BArray<BArray<DataBlockPos> >	oblockPositions;

	BUInt32			oblockSize;			///< The block size in use
	char*			oblock;				///< Current block data
	BUInt64			oblockPosition;			///< Current block position in file
	BUInt			opos;				///< Current data position within block
	BUInt			oblockNumberRead;		///< Current data block number when reading
	BUInt			oblockNumberWrite;		///< Current block number when writing fixed sized blocks
	BUInt			oblocketteNumber;		///< Current blockette number
	BUInt			ologNumber;

	// SEED information
	BDictString		ocodes;				///< List of coded text strings
	BDict<NameDesc>		ounitCodes;			///< List of coded unit names,descriptions

	// SEED import information
	BdsSeedType10		otype10;			///< Seed MetaData
	BdsSeedType11		otype11;
	BdsSeedType30		otype30;
	BdsSeedType52		otype52;
	BArray<BdsSeedType30>	oseedDataFormats;		///< Seed data formats
	BArray<BdsSeedStation>	oseedStations;			///< Seed stations from MetaData
	BArray<BdsSeedType52>	oseedChannels;			///< Seed channels from MetaData
	
	BArray<BString>		odataChannels;			///< The Seed data channels from data blocks
	BArray<int>		odataChannelFormats;		///< The data formats of each channel in libmseed form

	// SEED export information
	BArray<BInt32>		odataInt32;			///< Write data buffer
	BArray<BFloat32>	odataFloat32;			///< Write data buffer
	
	BUInt			odataChannel;			///< The current channel being written
	BUInt			odataSegment;			///< The current channel being written
	BUInt			odataTimeSpanBlock;		///< The block the time span list is in
	BDict<int>		ostationBlockNumbers;		///< The station block numbers
	BArray<BArray<BUInt32> >	odataStartBlocks;	///< Per data channel/segment start blocks
	BArray<BArray<BUInt32> >	odataEndBlocks;		///< Per data channel/segment end blocks

	static BMutex		olock;				///< Lock the libmseed library
	MSRecord*		omsr;				///< Libmseed data block processing
	
	BUInt			oabbrevCode;			///< The current abbreviation code
	BUInt			oabbrevUnitsCode;		///< The current units abbreviation code
};

}
#endif
