/*******************************************************************************
 *	TapeDigitiserDataFile.h	TapeDigitiserDataFile
 *			T.Barnaby,	BEAM Ltd,	2006-03-17
 *******************************************************************************
 */
#ifndef TapeDigitiserDataFile_H
#define TapeDigitiserDataFile_H	1

#include <BTypes.h>
#include <BError.h>
#include <BFile.h>
#include <BEntry.h>
#include <BArray.h>
#include <BTimeStampMs.h>

enum FileHeaderType { FileHeaderType_Standard = 1, FileHeaderType_TapeDigitiser = 10 };
enum FileSampleType { FileSampleType_Unknown, FileSampleType_Float32, FileSampleType_Float64, FileSampleType_Int16, FileSampleType_Int32 };

struct FileDataHeaderTape {
	uint32_t	type;			///< Type of file header
	uint32_t	headerLength;		///< Header length in bytes
	uint32_t	dataLength;		///< Data length in bytes
	uint32_t	dataType;		///< The type of a data sample
	uint32_t	numChannels;		///< The number of channels of data
	uint32_t	numSamples;		///< The number of samples of data per channel
	BTimeStampMs	startTime;		///< The time of the first sample
	BTimeStampMs	endTime;		///< The time of the last sample + 1
	BTimeStampMs	tapeLastVelaCode;	///< The last VELA code decoded
	float		tapeSignalLevelErrors;	///< The number of seconds the tape's FM signal was below a minimum level.
	float		tapeSpeedVariationErrors; ///< The number of seconds the FM carrier was above a defined maximum deviation
	float		tapeVelaBitErrors;	///< The number of VELA timecode bit errors per second.
	float		tapeFmSignalLevels[24];	///< The FM signal levels for each channel
};

struct FileData {
	FileDataHeaderTape	head;
	BArray<float>		data;
};

/// This class implements the TapeDigitiser's file output conversion and storeing system
class TapeDigitiserDataFile {
public:
	enum		{ ErrorEOF = 999 };
	
			TapeDigitiserDataFile();
			~TapeDigitiserDataFile();
	
	BError		open(BString fileName, BString mode);		///< Open the file for reading or writing
	void		close();					///< Close the file

	BError		init();						///< Initialise parameters from file
	uint32_t	blockSize();					///< Returns the data block size in bytes
	uint32_t	blockNumSamples();				///< Returns the number of samples of data in a block
	uint64_t	numSamples();					///< The number of samples in the file

	BError		readHeader(BEntryList& header);			///< Read the file header info
	BError		readDataBlock(uint32_t blockNumber, FileData& data);	///< Read a data block from the file

	BError		writeHeader(BEntryList& header);			///< Write the file header info
	BError		writeDataBlock(FileData& data);	///< Write a data block to the file
	BError		writeDataBlock(FileDataHeaderTape& head, float* data);	///< Write a data block to the file

private:
	BError		readDataBlock_1v1(uint32_t blockNumber, FileData& data);	///< Read a data block from the file
	BError		readDataBlock_2v0(uint32_t blockNumber, FileData& data);	///< Read a data block from the file
	uint32_t	computeChecksum(void* data, int nBytes);
	BString		ofileName;
	FILE*		ofile;
	BEntryList	oheader;
	off64_t		oheaderSize;
	uint32_t	oblockSize;
	uint32_t	oblockNumSamples;
	BString		oversion;
};

#endif
