/*******************************************************************************
 *	TimeControl.h	TimeControl system
 *			T.Barnaby,	BEAM Ltd,	2005-12-06
 *******************************************************************************
 */
#ifndef TIMECONTROL_H
#define TIMECONTROL_H	1

#include <BDspFifo.h>
#include <BDspFilterFir.h>
#include <BTimeStampMs.h>

#define DBG_TIMECODE		0x001000
#define DBG_TIMECODE_DETAIL1	0x002000
#define DBG_TIMECODE_DETAIL2	0x004000
#define DBG_TIMECODE_OUT	0x008000


/// A single Vela pulse
class Pulse {
public:
	enum	Type	{ Error, Start, One, Zero };			///< Pulse types
	
		Pulse(BUInt mark = 0, BUInt space = 0);
	
	Type	type(BUInt numBDspSamples);				///< Type of pulse
	void	clear();
	
	BUInt	omark;							///< Number of mark samples
	BUInt	ospace;							///< Number of space samples
};

/// The Time processing class
class TimeControl {
public:
	enum VelaBitRet	{ Data, MoreDataNeeded, Start, Error };

			TimeControl(BDspFifo& inputFifo, int sampleDelay, double samplerate);
			~TimeControl();

	void		clear();					///< Clears the Time Control engine
	void		config(int manual, BTimeStampMs startTime, int useVelaYear);	///< Set the mode and initial start time. Note in auto mode the year is taken from this.
	BError		process(BUInt numSamples, BUInt& bitErrors);	///< Processes the next block of samples, returns the number of bit errors

	BTimeStampMs	timeStamp(uint64_t sampleNumber);		///< Returns a timestamp for the given sample number
	BTimeStampMs	lastVelaCode(uint64_t sampleNumber);		///< Returns the last VELA code decoded with sample number referenced to absolute sample number

	double		velaQuality();					///< Indicator of VELA Quality 0 - 100
	float		realSampleRate();				///< Returns the calculated sample rate of the original "real-time" data

	void		debugPrint();					///< Print some debug status
	uint64_t	getSampleNumber();				///< returns TimeControl sample number
private:
	int			appendPulseTrain(BDspSample* data, BUInt numBDspSamples);///< Convert the set of samples to a mark/space pulse train and append. Returns the number of bit errors
	BError			velaProcess();						///< Perform the Vela timecode processing
	TimeControl::VelaBitRet	velaProcessBit(BIter& i, uint64_t& s, BUInt& v);	///< Process a single Vela bit
	int			velaProcessDigit(BIter& i, uint64_t& s, BUInt& v, int last = 0);	///< Process a Vela Digit
	void			velaReset(BIter& i, uint64_t& s);			///< Eeset Vela processing due to bit error
	void			velaPulseTrainPrint();
	BError			validateVela(BTimeStampMs time, BTimeStampMs timePrev);	///< Validate a timestamp against last one
	
	double			osampleRate;				///< The raw sample rate 
	BDspFifo		ofifo;					///< Internal FIFO
	BDspFilterFirFifo	oinputFilter;				///< Input Filter
	BDspFifoReader		oinputReader;				///< The Fifo reader for the input Fifo

	int			omanual;				///< Manual operation mode
	BTimeStampMs		ostartTime;				///< Start time for manual mode, in auto mode year is used
	int			ouseVelaYear;				///< Use the VELA year digit to make up the year
	int			ouseVelaDay;				///< Use the VELA day digit to make up the year
	
	int			ovelaYear;				///< The current Vela year
	int			ovelaDay;				///< The current Vela year day
	int			ovelaValid;				///< Indicates the time code is valid
	BTimeStampMs		ovelaTime;				///< The last valid Vela timestamp received
	uint64_t		ovelaTimeSample;			///< The absolute sample of the VelaTime

	BDspSample*		odata;					///< Sample data buffer

	uint64_t		opulseTrainSampleNumber;		///< The samples number of the first pulse in the pulse train
	BDspSample		olastBDspSample;
	Pulse			opulse;					///< The current pulse being created
	BList<Pulse>		opulseTrain;				///< The pulse train for the samples

	uint64_t		ototalSamples;				///< The total number of samples processed in valid VELA pulses
	BUInt			ototalBits;				///< The total number of valid vela bits found
	float			orealSampleRate;			///< The calculated sample rate of the original "real-time" data
	
	int			oinputSampleDelay;			///< The input sample delay due to any preprocess filters
	int			ovelaYearWrap;				///< The VELA day has gone beyond the end of year. Ignore VELA year after this.
};

#endif
