/*******************************************************************************
 *	TimeControlPll.h	TimeControl system
 *			T.Barnaby,	BEAM Ltd,	2009-10-06
 *******************************************************************************
 */
#ifndef TimeControlPll_H
#define TimeControlPll_H	1

#include <TimeControl.h>
#include <BDspDcRemove.h>
#include <VelaPll.h>

/// The Time processing class
class TimeControlPll {
public:
	enum VelaBitReq	{ Start, Data };
	enum VelaBitRet	{ OK, MoreDataNeeded, Error };

			TimeControlPll(BDspFifo& inputFifo, int sampleDelay, double samplerate);
			~TimeControlPll();

	void		clear();					///< Clears the Time Control engine
	void		config(TimecodeType type, BTimeStampMs startTime, int useVelaYear, int invert, int usePll, int keepGoing, int ignoreTimeJumps);	///< Set the mode and initial start time. Note in auto mode the year is taken from this.
	BError		process(BUInt numSamples, BUInt& bitErrors);	///< Processes the next block of samples, returns the number of bit errors

	BTimeStampMs	timeStamp(int64_t sampleNumber);		///< Returns a timestamp for the given sample number
	BTimeStampMs	lastVelaCode(uint64_t sampleNumber);		///< Returns the last VELA code decoded with sample number referenced to absolute sample number

	float		realSampleRate();				///< Returns the calculated sample rate of the original "real-time" data
	BUInt		totalBits();					///< Returns the total number of VELA bits processed
	BUInt		totalBitErrors();				///< Returns the total number of VELA bits errors
	BUInt		totalVelaCodes();				///< Returns the total number of VELA codes
	double		velaQuality();					///< Indicator of VELA Quality 0 - 1

	void		appendPulse(VelaPulse pulse);			///< Append a pulse
	VelaPll&	velaPll();					///< Returns a reference to the VELA PLL

private:
	BError			velaProcess();						///< Perform the Vela timecode processing
	VelaBitRet		velaProcessDigit(BIter& i, BUInt& v, int last = 0);	///< Process a Vela Digit
	VelaBitRet		velaProcessBit(BIter& i, VelaBitReq r, BUInt& v);
	void			velaClean(BIter& i);					///< Clean up VELA pulses
	void			velaError(BIter& i);					///< Reset Vela processing processing and bit errors
	BError			validateVela(BTimeStampMs time, BTimeStampMs timePrev);	///< Validate a timestamp against last one

	BError			hutchinsProcess();			///< Perform the Hutchins timecode processing
	VelaBitRet		hutchinsProcessDigit(BIter& i, BUInt& v, int start = 0);	///< Process a Vela Digit
	VelaBitRet		hutchinsProcessBit(BIter& i, VelaBitReq r, BUInt& v);
	
	TimecodeType		otype;					///< Timecode type
	int			oinvert;				///< Invert the timecode track
	int			ousePll;				///< Use PLL to process
	int			okeepGoing;				///< Keep going even if no Valid Timecode
	int			oignoreTimeJumps;			///< Ignore jumps in time, keep with interpolated time
	BList<VelaPulse>	opulses;				///< The VELA pulse list
	BUInt			opulsesOffset;				///< The offset in seconds to the first opulse
	int			oinputSampleDelay;			///< The input sample delay due to any preprocess filters
	double			osampleRate;				///< The raw sample rate 
	BDspFifo		ofifo1;					///< Internal FIFO
	BDspFilterFirFifo	oinputFilter;				///< Input Filter
	VelaPll			opll;					///< VELA PLL to VELA second pulse

	BTimeStampMs		ostartTime;				///< Start time for manual mode, in auto mode year is used
	int			ovelaYearWrap;				///< The VELA day has gone beyond the end of year. Ignore VELA year after this.
	int			ouseVelaYear;				///< Use the VELA year digit to make up the year
	int			ouseVelaDay;				///< Use the VELA day digit to make up the year
	
	int			ovelaYear;				///< The current Vela year
	int			ovelaDay;				///< The current Vela year day
	int			ovelaValid;				///< Indicates the time code is valid
	BTimeStampMs		ovelaTimePrev;				///< The previous valid Vela timestamp received
	BTimeStampMs		ovelaTime;				///< The last valid Vela timestamp received
	uint64_t		ovelaTimeSample;			///< The absolute sample of the VelaTime
	BIter			olastValidPulse;			///< The last VelaPulse processed
	BIter			onextValidPulse;			///< The first valid VelaPulse

	int			ofirst;					///< First set of samples
	int			osynchronised;				///< At VELA start pulse
	BDspSample*		odata;					///< Sample data buffer

	double			ovelaSamples;				///< The total number of samples processed in last VELA code
	BUInt			ovelaBits;				///< The number of bits in the last VELA code

	double			ototalSamples;				///< The total number of samples processed in valid VELA pulses
	BUInt			ototalBits;				///< The total number of valid vela bits found
	BUInt			ototalBitErrors;			///< The number of bit errors
	BUInt			ototalVelaCodes;			///< The number of VELA codes
	float			orealSampleRate;			///< The calculated sample rate of the original "real-time" data
};

#endif
