/*******************************************************************************
 *	VelaPll.h	Phase Locked Loop processing
 *			T.Barnaby,	BEAM Ltd,	2008-10-08
 *******************************************************************************
 */
#ifndef VelaPll_H
#define VelaPll_H	1

#include <BArray.h>
#include <BFifo.h>
#include <BDspFifo.h>
#include <BDspFilterFir.h>

enum	TimecodeType	{ TimecodeManual, TimecodeVela, TimecodeHutchins };	///< The timecode type

/// A single Vela pulse
class VelaPulse {
public:
	enum	Type		{ Error, Start, One, Zero };		///< Pulse types
	
		VelaPulse(TimecodeType timecodeType = TimecodeManual, double sampleRate = 100.0, BUInt64 sampleNumber = 0, BUInt64 endSampleNumber = 0, BUInt mark = 0, BUInt space = 0, double frequency = 0);
	
	Type	type();							///< Type of pulse
	void	clear();

	TimecodeType	otimecodeType;					///< The timecode type
	double		osampleRate;					///< The sample rate
	BUInt64		osampleNumber;					///< The sample number
	BUInt64		oendSampleNumber;				///< The end sample number
	BUInt		omark;						///< Number of mark samples
	BUInt		ospace;						///< Number of space samples
	double		ofrequency;					///< Current frequency
};

/// Frequency output filter
class VelaFreqFilter {
public:
			VelaFreqFilter(BDspFilterDesign design);

	void		clear();
	
	void		write(BDspSample sample);
	BDspSample	read();
private:
	BDspFifo		ofifo0;
	BDspFifo		ofifo1;
	BDspFilterFirFifo	ofilter;
	BDspFifoReader		ofifoReader;
	BUInt			oprevNum;
};

/// Frequency output filter
class VelaLoopFilter {
public:
			VelaLoopFilter();

	void		clear();
	
	BDspSample	process(BDspSample sample);
private:
	double		oxv0;
	double		oxv1;
	double		oyv0;
	double		oyv1;
};

/// The PLL class
class VelaPll {
public:
				VelaPll(BDspFifo& input, BList<VelaPulse>& pulses, double sampleRate);
				~VelaPll();

	BError			init(TimecodeType timecodeType);
	void			clear();
	
	int			process(BUInt numSamples, int invert, int usePll, BUInt& numSamplesProcessed, BUInt& bitErrors);		///< Processes the next block of samples, returns the number of bits
	double			pllFrequency();
	
	void			setGain(double gain);
	
private:
	int			processData(const BDspSample* data, BUInt numSamples, int invert, BUInt& bitErrors);
	int			processDataSimple(const BDspSample* data, BUInt numSamples, int invert, BUInt& bitErrors);

	TimecodeType		otimecodeType;				///< The timecode type
	BDspFifoReader		oinputReader;
	BList<VelaPulse>&	opulses;				///< List of Vela Pulses
	double			osampleRate;				///< The raw sample rate
	VelaLoopFilter		oloopFilter;
	VelaFreqFilter		ofrequencyFilter;
	double			opllGain;				///< The PLL loop gain
	double			olimit;					///< PLL frequency range limit
	BArray<BDspSample>	oref;					///< The PLL reference signal
	double			opllFrequency;				///< Current frequency of PLL
	double			opllFrequencyFiltered;			///< Current frequency of PLL, filtered
	double			opllPhase;				///< Current PLL phase
	BDspSample		osample;				///< Current sample value (half VELA frequency)
	BDspSample		olastSample;				///< Previous sample value
	uint64_t		osampleNumber;				///< Current sample number
	uint64_t		olastPulseSampleNumber;			///< The last Vela Pulses start sample number
	uint64_t		olastTransition;			///< Last Vela pulse transition sample number
	double			ofrequency;				///< Filtered current frequency of PLL
	BUInt			omark;					///< Current Vela Pulse mark count
	BUInt			ospace;					///< Current Vela Pulse space count
};

#endif
